const width = 960;
const height = 500;
const margin = { top: 80, right: 40, bottom: 90, left: 90 };
const xAxisLabelOffset = 60;
const yAxisLabelOffset = 60;
const yAxisTextOffset = 20;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const horizontalTicks = 10;
const circleRadiusMin = 3.5;
const circleRadiusMax = 15;

const svg = d3.select('#scatter-plot-container');

svg.attr('height', height).attr('width', width);

const body = d3.select('#scatter-plot-body');
body.attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');
const csvUrl =
  'https://gist.githubusercontent.com/rohame/c66da4d5fb31f35bf2a02d6417a38f64/raw/combined_logan_data.csv';

let data = [];
const loadData = () => {
  return Promise.all([d3.csv(csvUrl)]).then(datasets => {
    data = datasets[0];
    data = data.map(d => {
      d.DATE = d.DATE.slice(0, -3);
      return {
        date: new Date(d.DATE),
        TMAX: +d.TMAX,
        PRCP: +d.PRCP,
        DEP_DELAY: +d.WEATHER_DELAY,
        year: d.YEAR
      };
    });
    return data;
  });
};

const showTitle = () => {
  svg
    .append('text')
    .text('Logan Airport Weather vs Delay')
    .attr('transform', 'translate(' + width / 2 + ',40)')
    .attr('font-size', '3em');
};

const showLabel = () => {
  body
    .append('text')
    .text('TMAX')
  	.attr("class","axis-label")
    .attr('transform', 'translate(' + -yAxisLabelOffset + ','+innerHeight/2+') rotate(-90)');

body
    .append('text')
    .text('Date')
  	.attr("class","axis-label")
    .attr('transform', 'translate(' + innerWidth/2 + ','+(innerHeight+xAxisLabelOffset)+')');
};


const showData = () => {
  showTitle();

  showLabel();

  const leftScale = d3
    .scaleLinear()
    .domain(d3.extent(data, d => d.TMAX))
    .range([innerHeight, 0]);

  const xScale = d3
    .scaleTime()
    .domain(d3.extent(data, d => d.date))
    .range([0, innerWidth]);

  const sizeScale = d3
    .scaleLinear()
    .domain(d3.extent(data, d => d.DEP_DELAY))
    .range([circleRadiusMin, circleRadiusMax]);

  const xAxis = d3
    .axisBottom(xScale)
    .tickFormat(d3.timeFormat('%b'))
    .tickSize(-innerHeight);

  const leftAxis = d3
    .axisLeft(leftScale)
    .tickSize(-innerWidth)
    .ticks(horizontalTicks);

  body
    .append('g')
    .call(leftAxis)
    .selectAll('.domain')
    .remove();

  d3.selectAll('.tick')
    .select('text')
    .attr('transform', 'translate(' + -yAxisTextOffset + ',0)')
    .attr('dy', '.32em');

  body
    .append('g')
    .attr('transform', 'translate(0,' + innerHeight + ')')
    .call(xAxis)
    .selectAll('.domain')
    .remove();

  let cScale = d3
    .scaleLinear()
    .domain(d3.extent(data, d => d.PRCP))
    .range([0, 1]);

  let colorMap = d3.interpolateRgb(d3.rgb('#55ffff'), d3.rgb('#3399ff'));

  let weatherContainer = body.append('g').attr('id', 'weather-data');
  weatherContainer
    .selectAll('.dot')
    .data(data)
    .enter()
    .append('circle')
    .attr('r', d => sizeScale(d.DEP_DELAY))
    .attr('cx', d => xScale(d.date))
    .attr('cy', d => leftScale(d.TMAX))
    .attr('fill', d => {
      return d.PRCP < 0.1 ? 'rgba(224,224,240,0.5)' : colorMap(cScale(d.PRCP));
    })
  	.append("title").text(d=>d.DEP_DELAY);
};

loadData().then(showData);
